@@ -789,7 +789,7 @@ static apr_status_t session_start(h2_proxy_session *session)
     apr_socket_t *s;
     
     s = ap_get_conn_socket(session->c);
-#if (!defined(WIN32) && !defined(NETWARE)) || defined(DOXYGEN)
+#if !defined(WIN32) && !defined(NETWARE)
     if (s) {
         ap_sock_disable_nagle(s);
     }
@@ -850,6 +850,18 @@ static apr_status_t open_stream(h2_proxy_session *session, const char *url,
     dconf = ap_get_module_config(r->per_dir_config, &proxy_module);
     if (dconf->preserve_host) {
         authority = orig_host;
+        if (!authority) {
+            /* Duplicate mod_proxy behaviour if ProxyPreserveHost is
+             * used but an "HTTP/0.9" request is received without a
+             * Host: header */
+            authority = r->server->server_hostname;
+            ap_log_rerror(APLOG_MARK, APLOG_WARNING, 0, r, APLOGNO(10511)
+                          "HTTP/0.9 request (with no host line) "
+                          "on incoming request and preserve host set "
+                          "forcing hostname to be %s for uri %s",
+                          authority, r->uri);
+            apr_table_setn(r->headers_in, "Host", authority);
+        }
     }
     else {
         authority = puri.hostname;
@@ -1681,17 +1693,7 @@ static int done_iter(void *udata, void *val)
     h2_proxy_stream *stream = val;
     int touched = (stream->data_sent || stream->data_received ||
                    stream->id <= ctx->session->last_stream_id);
-    if (touched && stream->output) {
-      apr_bucket *b = ap_bucket_error_create(HTTP_BAD_GATEWAY, NULL,
-                                             stream->r->pool,
-                                             stream->cfront->bucket_alloc);
-      APR_BRIGADE_INSERT_TAIL(stream->output, b);
-      b = apr_bucket_eos_create(stream->cfront->bucket_alloc);
-      APR_BRIGADE_INSERT_TAIL(stream->output, b);
-      ap_pass_brigade(stream->r->output_filters, stream->output);
-    }
-    ctx->done(ctx->session, stream->r, APR_ECONNABORTED, touched,
-              stream->error_code);
+    ctx->done(ctx->session, stream->r, APR_ECONNABORTED, touched, stream->error_code);
     return 1;
 }
 
