@@ -63,7 +63,19 @@ class TestShutdown:
         assert env.apache_restart() == 0
         url = env.mkurl("https", "test1", "/index.html")
         for i in range(7):
-            r = env.curl_get(url, options=['-vvv'])
-            assert r.exit_code == 0, f"failed on {i}. request: {r.stdout} {r.stderr}"
-            assert r.response["status"] == 200
-            assert "HTTP/2" == r.response["protocol"]
\ No newline at end of file
+            r = env.curl_get(url, options=['-v'])
+            # requests should succeed, but rarely connections get closed
+            # before the response is received
+            if r.exit_code in [16, 55]:
+                # curl send error
+                assert r.response is None
+            else:
+                assert r.exit_code == 0, f"failed on {i}. request: {r.stdout} {r.stderr}"
+                assert r.response["status"] == 200
+                assert "HTTP/2" == r.response["protocol"]
+        #
+        env.httpd_error_log.ignore_recent(
+            lognos = [
+                "AH03490"   # scoreboard is full, not at MaxRequestWorkers
+            ]
+        )
\ No newline at end of file
